import asyncio
import unittest

from functools import wraps


def run_until_complete(fun):
    if not asyncio.iscoroutinefunction(fun):
        fun = asyncio.coroutine(fun)

    @wraps(fun)
    def wrapper(test, *args, **kw):
        loop = test.loop
        ret = loop.run_until_complete(
            asyncio.wait_for(fun(test, *args, **kw), 15, loop=loop))
        return ret
    return wrapper


class BaseTest(unittest.TestCase):
    """Base test case for unittests.
    """
    def setUp(self):
        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(None)

    def tearDown(self):
        self.loop.close()
        del self.loop



# _converters_to_bytes_map = {
#     bytes: lambda val: val,
#     bytearray: lambda val: val,
#     str: lambda val: val.encode('utf-8'),
#     int: lambda val: str(val).encode('utf-8'),
#     float: lambda val: str(val).encode('utf-8'),
#     }
#
#
# _converters_to_str_map = {
#     str: lambda val: val,
#     bytearray: lambda val: bytes(val).decode('utf-8'),
#     bytes: lambda val: val.decode('utf-8'),
#     int: lambda val: str(val),
#     float: lambda val: str(val),
#     }
#
#
# def _convert_to_bytes(value):
#     if type(value) in _converters_to_bytes_map:
#         converted_value = _converters_to_bytes_map[type(value)](value)
#     else:
#         raise TypeError("Argument {!r} expected to be of bytes,"
#                         " str, int or float type".format(value))
#     return converted_value
#
#
# def _convert_to_str(value):
#     if type(value) in _converters_to_str_map:
#         converted_value = _converters_to_str_map[type(value)](value)
#     else:
#         raise TypeError("Argument {!r} expected to be of bytes,"
#                         " str, int or float type".format(value))
#     return converted_value



from tests._testutils import run_until_complete
from tests.base import AIOPyMySQLTestCase


class TestExample(AIOPyMySQLTestCase):
    @run_until_complete
    def test_example(self):
        conn = yield from self.connect(db='mysql')

        cur = yield from conn.cursor()
        yield from cur.execute("SELECT Host,User FROM user")
        # print(cur.description)
        r = yield from cur.fetchall()
        self.assertTrue(r)
        # print(r)
        yield from cur.close()
        conn.close()

    @run_until_complete
    def test_pool_example(self):
        pool = yield from self.create_pool(db='mysql')

        with (yield from pool) as conn:
            cur = yield from conn.cursor()
            yield from cur.execute("SELECT 10")
            # print(cur.description)
            (r,) = yield from cur.fetchone()
            self.assertTrue(r, 10)
        pool.close()
        yield from pool.wait_closed()

    @run_until_complete
    def test_callproc(self):

        conn = yield from self.connect(db='mysql')

        cur = yield from conn.cursor()

        yield from cur.execute("DROP PROCEDURE IF EXISTS myinc;")
        yield from cur.execute("""CREATE PROCEDURE myinc(p1 INT)
                               BEGIN
                                   SELECT p1 + 1;
                               END
                               """)

        yield from cur.callproc('myinc', [1])
        (ret, ) = yield from cur.fetchone()
        assert 2, ret

        yield from cur.close()
        conn.close()



import asyncio
import aiomysql
from tests import base
from tests._testutils import run_until_complete


class TestBulkInserts(base.AIOPyMySQLTestCase):
    cursor_type = aiomysql.cursors.DictCursor

    def setUp(self):
        super(TestBulkInserts, self).setUp()

        @asyncio.coroutine
        def prepare(self):
            # create a table ane some data to query
            self.conn = conn = self.connections[0]
            c = yield from conn.cursor(self.cursor_type)
            yield from c.execute("drop table if exists bulkinsert;")
            yield from c.execute(
                """CREATE TABLE bulkinsert
                (
                id int(11),
                name char(20),
                age int,
                height int,
                PRIMARY KEY (id)
                )
                """)

        self.loop.run_until_complete(prepare(self))

    @asyncio.coroutine
    def _verify_records(self, data):
        conn = self.connections[0]
        cursor = yield from conn.cursor()
        yield from cursor.execute(
            "SELECT id, name, age, height from bulkinsert")
        result = yield from cursor.fetchall()
        yield from cursor.execute('commit')
        self.assertEqual(sorted(data), sorted(result))

    @run_until_complete
    def test_bulk_insert(self):
        conn = self.connections[0]
        cursor = yield from conn.cursor()

        data = [(0, "bob", 21, 123), (1, "jim", 56, 45), (2, "fred", 100, 180)]
        yield from cursor.executemany(
            "insert into bulkinsert (id, name, age, height) "
            "values (%s,%s,%s,%s)", data)
        self.assertEqual(
            cursor._last_executed, bytearray(
                b"insert into bulkinsert (id, name, age, height) values "
                b"(0,'bob',21,123),(1,'jim',56,45),(2,'fred',100,180)"))
        yield from cursor.execute('commit')
        yield from self._verify_records(data)

    @run_until_complete
    def test_bulk_insert_multiline_statement(self):
        conn = self.connections[0]
        cursor = yield from conn.cursor()
        data = [(0, "bob", 21, 123), (1, "jim", 56, 45), (2, "fred", 100, 180)]
        yield from cursor.executemany("""insert
            into bulkinsert (id, name,
            age, height)
            values (%s,
            %s , %s,
            %s )
             """, data)
        self.assertEqual(cursor._last_executed.strip(), bytearray(b"""insert
            into bulkinsert (id, name,
            age, height)
            values (0,
            'bob' , 21,
            123 ),(1,
            'jim' , 56,
            45 ),(2,
            'fred' , 100,
            180 )"""))
        yield from cursor.execute('commit')
        yield from self._verify_records(data)

    @run_until_complete
    def test_bulk_insert_single_record(self):
        conn = self.connections[0]
        cursor = yield from conn.cursor()
        data = [(0, "bob", 21, 123)]
        yield from cursor.executemany(
            "insert into bulkinsert (id, name, age, height) "
            "values (%s,%s,%s,%s)", data)
        yield from cursor.execute('commit')
        yield from self._verify_records(data)

    @run_until_complete
    def test_insert_on_duplicate_key_update(self):
        # executemany should work with "insert ... on update" "
        conn = self.connections[0]
        cursor = yield from conn.cursor()
        data = [(0, "bob", 21, 123), (1, "jim", 56, 45), (2, "fred", 100, 180)]
        yield from cursor.executemany("""insert
            into bulkinsert (id, name,
            age, height)
            values (%s,
            %s , %s,
            %s ) on duplicate key update
            age = values(age)
             """, data)
        self.assertEqual(cursor._last_executed.strip(), bytearray(b"""insert
            into bulkinsert (id, name,
            age, height)
            values (0,
            'bob' , 21,
            123 ),(1,
            'jim' , 56,
            45 ),(2,
            'fred' , 100,
            180 ) on duplicate key update
            age = values(age)"""))
        yield from cursor.execute('commit')
        self._verify_records(data)



import asyncio
import datetime

import aiomysql.cursors
from tests import base
from tests._testutils import run_until_complete


class TestDictCursor(base.AIOPyMySQLTestCase):
    bob = {'name': 'bob', 'age': 21,
           'DOB': datetime.datetime(1990, 2, 6, 23, 4, 56)}
    jim = {'name': 'jim', 'age': 56,
           'DOB': datetime.datetime(1955, 5, 9, 13, 12, 45)}
    fred = {'name': 'fred', 'age': 100,
            'DOB': datetime.datetime(1911, 9, 12, 1, 1, 1)}

    cursor_type = aiomysql.cursors.DictCursor

    def setUp(self):
        super(TestDictCursor, self).setUp()
        self.conn = conn = self.connections[0]

        @asyncio.coroutine
        def prepare():
            c = yield from conn.cursor(self.cursor_type)

            # create a table ane some data to query
            yield from c.execute("drop table if exists dictcursor")
            yield from c.execute(
                """CREATE TABLE dictcursor (name char(20), age int ,
                DOB datetime)""")
            data = [("bob", 21, "1990-02-06 23:04:56"),
                    ("jim", 56, "1955-05-09 13:12:45"),
                    ("fred", 100, "1911-09-12 01:01:01")]
            yield from c.executemany("insert into dictcursor values "
                                     "(%s,%s,%s)",
                                     data)

        self.loop.run_until_complete(prepare())

    def tearDown(self):
        @asyncio.coroutine
        def shutdown():
            c = yield from self.conn.cursor()
            yield from c.execute("drop table dictcursor;")

        self.loop.run_until_complete(shutdown())
        super(TestDictCursor, self).tearDown()

    @run_until_complete
    def test_dictcursor(self):
        bob, jim, fred = self.bob.copy(), self.jim.copy(), self.fred.copy()
        # all assert test compare to the structure as would come
        # out from MySQLdb
        conn = self.conn
        c = yield from conn.cursor(self.cursor_type)

        # try an update which should return no rows
        yield from c.execute("update dictcursor set age=20 where name='bob'")
        bob['age'] = 20
        # pull back the single row dict for bob and check
        yield from c.execute("SELECT * from dictcursor where name='bob'")
        r = yield from c.fetchone()
        self.assertEqual(bob, r, "fetchone via DictCursor failed")
        # same again, but via fetchall => tuple)
        yield from c.execute("SELECT * from dictcursor where name='bob'")
        r = yield from c.fetchall()
        self.assertEqual([bob], r,
                         "fetch a 1 row result via fetchall failed via "
                         "DictCursor")

        # get all 3 row via fetchall
        yield from c.execute("SELECT * from dictcursor")
        r = yield from c.fetchall()
        self.assertEqual([bob, jim, fred], r, "fetchall failed via DictCursor")

        # get all 2 row via fetchmany
        yield from c.execute("SELECT * from dictcursor")
        r = yield from c.fetchmany(2)
        self.assertEqual([bob, jim], r, "fetchmany failed via DictCursor")
        yield from c.execute('commit')

    @run_until_complete
    def test_custom_dict(self):
        class MyDict(dict):
            pass

        class MyDictCursor(self.cursor_type):
            dict_type = MyDict

        keys = ['name', 'age', 'DOB']
        bob = MyDict([(k, self.bob[k]) for k in keys])
        jim = MyDict([(k, self.jim[k]) for k in keys])
        fred = MyDict([(k, self.fred[k]) for k in keys])

        cur = yield from self.conn.cursor(MyDictCursor)
        yield from cur.execute("SELECT * FROM dictcursor WHERE name='bob'")
        r = yield from cur.fetchone()
        self.assertEqual(bob, r, "fetchone() returns MyDictCursor")

        yield from cur.execute("SELECT * FROM dictcursor")
        r = yield from cur.fetchall()
        self.assertEqual([bob, jim, fred], r,
                         "fetchall failed via MyDictCursor")

        yield from cur.execute("SELECT * FROM dictcursor")
        r = yield from cur.fetchmany(2)
        self.assertEqual([bob, jim], r,
                         "list failed via MyDictCursor")

    @run_until_complete
    def test_ssdictcursor(self):
        conn = self.conn
        c = yield from conn.cursor(aiomysql.cursors.SSDictCursor)
        yield from c.execute("SELECT * from dictcursor where name='bob'")
        r = yield from c.fetchall()
        self.assertEqual([self.bob], r,
                         "fetch a 1 row result via fetchall failed via "
                         "DictCursor")



import asyncio
from tests import base
from tests._testutils import run_until_complete

from aiomysql import ProgrammingError, Cursor


class TestCursor(base.AIOPyMySQLTestCase):

    @asyncio.coroutine
    def _prepare(self, conn):
        cur = yield from conn.cursor()
        yield from cur.execute("DROP TABLE IF EXISTS tbl;")

        yield from cur.execute("""CREATE TABLE tbl (
                 id MEDIUMINT NOT NULL AUTO_INCREMENT,
                 name VARCHAR(255) NOT NULL,
                 PRIMARY KEY (id));""")

        for i in [(1, 'a'), (2, 'b'), (3, 'c')]:
            yield from cur.execute("INSERT INTO tbl VALUES(%s, %s)", i)
        yield from cur.execute("DROP TABLE IF EXISTS tbl2")
        yield from cur.execute("""CREATE TABLE tbl2
                                  (id int, name varchar(255))""")
        yield from conn.commit()

    @asyncio.coroutine
    def _prepare_procedure(self, conn):
        cur = yield from conn.cursor()
        yield from cur.execute("DROP PROCEDURE IF EXISTS myinc;")
        yield from cur.execute("""CREATE PROCEDURE myinc(p1 INT)
                               BEGIN
                                   SELECT p1 + 1;
                               END
                               """)
        yield from conn.commit()

    @run_until_complete
    def test_description(self):
        conn = self.connections[0]
        yield from self._prepare(conn)
        cur = yield from conn.cursor()
        self.assertEqual(None, cur.description)
        yield from cur.execute('SELECT * from tbl;')

        self.assertEqual(len(cur.description), 2,
                         'cursor.description describes too many columns')

        self.assertEqual(len(cur.description[0]), 7,
                         'cursor.description[x] tuples must have '
                         '7 elements')

        self.assertEqual(cur.description[0][0].lower(), 'id',
                         'cursor.description[x][0] must return column '
                         'name')

        self.assertEqual(cur.description[1][0].lower(), 'name',
                         'cursor.description[x][0] must return column '
                         'name')

        # Make sure self.description gets reset, cursor should be
        # set to None in case of none resulting queries like DDL
        yield from cur.execute('DROP TABLE IF EXISTS foobar;')
        self.assertEqual(None, cur.description)

    @run_until_complete
    def test_cursor_properties(self):
        conn = self.connections[0]
        cur = yield from conn.cursor()
        self.assertIs(cur.connection, conn)
        cur.setinputsizes()
        cur.setoutputsizes()
        self.assertEqual(cur.echo, False)

    @run_until_complete
    def test_scroll_relative(self):
        conn = self.connections[0]
        yield from self._prepare(conn)
        cur = yield from conn.cursor()
        yield from cur.execute('SELECT * FROM tbl;')
        yield from cur.scroll(1)
        ret = yield from cur.fetchone()
        self.assertEqual((2, 'b'), ret)

    @run_until_complete
    def test_scroll_absolute(self):
        conn = self.connections[0]
        yield from self._prepare(conn)
        cur = yield from conn.cursor()
        yield from cur.execute('SELECT * FROM tbl;')
        yield from cur.scroll(2, mode='absolute')
        ret = yield from cur.fetchone()
        self.assertEqual((3, 'c'), ret)

    @run_until_complete
    def test_scroll_errors(self):
        conn = self.connections[0]
        cur = yield from conn.cursor()

        with self.assertRaises(ProgrammingError):
            yield from cur.scroll(2, mode='absolute')

        cur = yield from conn.cursor()
        yield from cur.execute('SELECT * FROM tbl;')

        with self.assertRaises(ProgrammingError):
            yield from cur.scroll(2, mode='not_valid_mode')

    @run_until_complete
    def test_scroll_index_error(self):
        conn = self.connections[0]
        yield from self._prepare(conn)
        cur = yield from conn.cursor()
        yield from cur.execute('SELECT * FROM tbl;')
        with self.assertRaises(IndexError):
            yield from cur.scroll(1000)

    @run_until_complete
    def test_close(self):
        conn = self.connections[0]
        cur = yield from conn.cursor()
        yield from cur.close()
        self.assertTrue(cur.closed)
        with self.assertRaises(ProgrammingError):
            yield from cur.execute('SELECT 1')
        # try to close for second time
        yield from cur.close()

    @run_until_complete
    def test_arraysize(self):
        conn = self.connections[0]
        cur = yield from conn.cursor()
        self.assertEqual(1, cur.arraysize)
        cur.arraysize = 10
        self.assertEqual(10, cur.arraysize)

    @run_until_complete
    def test_rows(self):
        conn = self.connections[0]
        yield from self._prepare(conn)

        cur = yield from conn.cursor()
        yield from cur.execute('SELECT * from tbl')
        self.assertEqual(3, cur.rowcount)
        self.assertEqual(0, cur.rownumber)
        yield from cur.fetchone()
        self.assertEqual(1, cur.rownumber)
        self.assertEqual(None, cur.lastrowid)
        yield from cur.execute('INSERT INTO tbl VALUES (%s, %s)', (4, 'd'))
        self.assertNotEqual(0, cur.lastrowid)
        yield from conn.commit()

    @run_until_complete
    def test_callproc(self):
        conn = yield from self.connect()
        yield from self._prepare_procedure(conn)
        cur = yield from conn.cursor()
        yield from cur.callproc('myinc', [1])
        ret = yield from cur.fetchone()
        self.assertEqual((2,), ret)
        yield from cur.close()
        with self.assertRaises(ProgrammingError):
            yield from cur.callproc('myinc', [1])
        conn.close()

    @run_until_complete
    def test_fetchone_no_result(self):
        # test a fetchone() with no rows
        conn = self.connections[0]
        c = yield from conn.cursor()
        yield from c.execute("create table test_nr (b varchar(32))")
        try:
            data = "pymysql"
            yield from c.execute("insert into test_nr (b) values (%s)",
                                 (data,))
            r = yield from c.fetchone()
            self.assertEqual(None, r)
        finally:
            yield from c.execute("drop table test_nr")

    @run_until_complete
    def test_fetchmany_no_result(self):
        conn = self.connections[0]
        cur = yield from conn.cursor()
        yield from cur.execute('DROP TABLE IF EXISTS foobar;')
        r = yield from cur.fetchmany()
        self.assertEqual([], r)

    @run_until_complete
    def test_fetchall_no_result(self):
        # test a fetchone() with no rows
        conn = self.connections[0]
        cur = yield from conn.cursor()
        yield from cur.execute('DROP TABLE IF EXISTS foobar;')
        r = yield from cur.fetchall()
        self.assertEqual([], r)

    @run_until_complete
    def test_fetchall_with_scroll(self):
        conn = self.connections[0]
        yield from self._prepare(conn)
        cur = yield from conn.cursor()
        yield from cur.execute('SELECT * FROM tbl;')
        yield from cur.scroll(1)
        ret = yield from cur.fetchall()
        self.assertEquals(((2, 'b'), (3, 'c')), ret)

    @run_until_complete
    def test_aggregates(self):
        """ test aggregate functions """
        conn = self.connections[0]
        c = yield from conn.cursor()
        try:
            yield from c.execute('create table test_aggregates (i integer)')
            for i in range(0, 10):
                yield from c.execute(
                    'insert into test_aggregates (i) values (%s)', (i,))
            yield from c.execute('select sum(i) from test_aggregates')
            r, = yield from c.fetchone()
            self.assertEqual(sum(range(0, 10)), r)
        finally:
            yield from c.execute('drop table test_aggregates')

    @run_until_complete
    def test_single_tuple(self):
        """ test a single tuple """
        conn = self.connections[0]
        c = yield from conn.cursor()
        try:
            yield from c.execute(
                "create table mystuff (id integer primary key)")
            yield from c.execute("insert into mystuff (id) values (1)")
            yield from c.execute("insert into mystuff (id) values (2)")
            yield from c.execute("select id from mystuff where id in %s",
                                 ((1,),))
            r = yield from c.fetchall()
            self.assertEqual([(1,)], list(r))
        finally:
            yield from c.execute("drop table mystuff")

    @run_until_complete
    def test_executemany(self):
        conn = self.connections[0]
        yield from self._prepare(conn)
        cur = yield from conn.cursor()
        self.assertEqual(None, cur.description)
        args = [1, 2, 3]
        row_count = yield from cur.executemany(
            'SELECT * FROM tbl WHERE id  = %s;', args)
        self.assertEqual(row_count, 3)
        r = yield from cur.fetchall()
        # TODO: if this right behaviour
        self.assertEquals(((3, 'c'),),  r)

        # calling execute many without args
        row_count = yield from cur.executemany('SELECT 1;', ())
        self.assertIsNone(row_count)

    @run_until_complete
    def test_custom_cursor(self):
        class MyCursor(Cursor):
            pass
        conn = self.connections[0]
        cur = yield from conn.cursor(MyCursor)
        self.assertIsInstance(cur, MyCursor)
        yield from cur.execute("SELECT 42;")
        (r, ) = yield from cur.fetchone()
        self.assertEqual(r, 42)

    @run_until_complete
    def test_custom_cursor_not_cursor_subclass(self):
        class MyCursor2:
            pass
        conn = self.connections[0]
        with self.assertRaises(TypeError):
            yield from conn.cursor(MyCursor2)



import asyncio
from aiomysql import sa
# from aiomysql.connection import TIMEOUT

import os
import unittest

from sqlalchemy import MetaData, Table, Column, Integer, String

meta = MetaData()
tbl = Table('sa_tbl3', meta,
            Column('id', Integer, nullable=False,
                   primary_key=True),
            Column('name', String(255)))


class TestEngine(unittest.TestCase):
    def setUp(self):
        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(None)
        self.host = os.environ.get('MYSQL_HOST', 'localhost')
        self.port = os.environ.get('MYSQL_PORT', 3306)
        self.user = os.environ.get('MYSQL_USER', 'root')
        self.db = os.environ.get('MYSQL_DB', 'test_pymysql')
        self.password = os.environ.get('MYSQL_PASSWORD', '')
        self.engine = self.loop.run_until_complete(self.make_engine())
        self.loop.run_until_complete(self.start())

    def tearDown(self):
        self.engine.terminate()
        self.loop.run_until_complete(self.engine.wait_closed())
        self.loop.close()

    @asyncio.coroutine
    def make_engine(self, use_loop=True, **kwargs):
        if use_loop:
            return (yield from sa.create_engine(db=self.db,
                                                user=self.user,
                                                password=self.password,
                                                host=self.host,
                                                loop=self.loop,
                                                **kwargs))
        else:
            return (yield from sa.create_engine(db=self.db,
                                                user=self.user,
                                                password=self.password,
                                                host=self.host,
                                                **kwargs))

    @asyncio.coroutine
    def start(self):
        with (yield from self.engine) as conn:
            yield from conn.execute("DROP TABLE IF EXISTS sa_tbl3")
            yield from conn.execute("CREATE TABLE sa_tbl3 "
                                    "(id serial, name varchar(255))")

    def test_dialect(self):
        self.assertEqual(sa.engine._dialect, self.engine.dialect)

    def test_name(self):
        self.assertEqual('mysql', self.engine.name)

    def test_driver(self):
        self.assertEqual('pymysql', self.engine.driver)

    # def test_dsn(self):
    #     self.assertEqual(
    #         'dbname=aiomysql user=aiomysql password=xxxxxx host=127.0.0.1',
    #         self.engine.dsn)

    def test_minsize(self):
        self.assertEqual(10, self.engine.minsize)

    def test_maxsize(self):
        self.assertEqual(10, self.engine.maxsize)

    def test_size(self):
        self.assertEqual(10, self.engine.size)

    def test_freesize(self):
        self.assertEqual(10, self.engine.freesize)

    def test_make_engine_with_default_loop(self):

        @asyncio.coroutine
        def go():
            engine = yield from self.make_engine(use_loop=False)
            engine.close()
            yield from engine.wait_closed()

        asyncio.set_event_loop(self.loop)
        try:
            self.loop.run_until_complete(go())
        finally:
            asyncio.set_event_loop(None)

    def test_not_context_manager(self):
        @asyncio.coroutine
        def go():
            with self.assertRaises(RuntimeError):
                with self.engine:
                    pass
        self.loop.run_until_complete(go())

    def test_release_transacted(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.engine.acquire()
            tr = yield from conn.begin()
            with self.assertRaises(sa.InvalidRequestError):
                self.engine.release(conn)
            del tr
        self.loop.run_until_complete(go())

    # def test_timeout(self):
    #     self.assertEqual(TIMEOUT, self.engine.timeout)

    # def test_timeout_override(self):
    #     @asyncio.coroutine
    #     def go():
    #         timeout = 1
    #         engine = yield from self.make_engine(timeout=timeout)
    #         self.assertEqual(timeout, engine.timeout)
    #         conn = yield from engine.acquire()
    #         with self.assertRaises(asyncio.TimeoutError):
    #             yield from conn.execute("SELECT pg_sleep(10)")
    #     self.loop.run_until_complete(go())

    def test_cannot_acquire_after_closing(self):
        @asyncio.coroutine
        def go():
            engine = yield from self.make_engine()
            engine.close()

            with self.assertRaises(RuntimeError):
                yield from engine.acquire()
            yield from engine.wait_closed()
        self.loop.run_until_complete(go())

    def test_wait_closed(self):
        @asyncio.coroutine
        def go():
            engine = yield from self.make_engine()

            c1 = yield from engine.acquire()
            c2 = yield from engine.acquire()
            self.assertEqual(10, engine.size)
            self.assertEqual(8, engine.freesize)

            ops = []

            @asyncio.coroutine
            def do_release(conn):
                yield from asyncio.sleep(0, loop=self.loop)
                engine.release(conn)
                ops.append('release')

            @asyncio.coroutine
            def wait_closed():
                yield from engine.wait_closed()
                ops.append('wait_closed')

            engine.close()
            yield from asyncio.gather(wait_closed(),
                                      do_release(c1),
                                      do_release(c2),
                                      loop=self.loop)
            self.assertEqual(['release', 'release', 'wait_closed'], ops)
            self.assertEqual(0, engine.freesize)
            engine.close()
            yield from engine.wait_closed()

        self.loop.run_until_complete(go())

    def test_terminate_with_acquired_connections(self):

        @asyncio.coroutine
        def go():
            engine = yield from self.make_engine()
            conn = yield from engine.acquire()
            engine.terminate()
            yield from engine.wait_closed()

            self.assertTrue(conn.closed)

        self.loop.run_until_complete(go())



import os
import re
import sys
from setuptools import setup, find_packages


install_requires = ['PyMySQL<=0.6.6']

PY_VER = sys.version_info

if PY_VER >= (3, 4):
    pass
elif PY_VER >= (3, 3):
    install_requires.append('asyncio')
else:
    raise RuntimeError("aiomysql doesn't suppport Python earllier than 3.3")


def read(f):
    return open(os.path.join(os.path.dirname(__file__), f)).read().strip()

extras_require = {'sa': ['sqlalchemy>=0.9'], }


def read_version():
    regexp = re.compile(r"^__version__\W*=\W*'([\d.abrc]+)'")
    init_py = os.path.join(os.path.dirname(__file__),
                           'aiomysql', '__init__.py')
    with open(init_py) as f:
        for line in f:
            match = regexp.match(line)
            if match is not None:
                return match.group(1)
        else:
            raise RuntimeError('Cannot find version in aiomysql/__init__.py')

classifiers = [
    'License :: OSI Approved :: MIT License',
    'Intended Audience :: Developers',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Operating System :: POSIX',
    'Environment :: Web Environment',
    'Development Status :: 3 - Alpha',
    'Topic :: Database',
    'Topic :: Database :: Front-Ends',
]


setup(name='aiomysql',
      version=read_version(),
      description=('MySQL driver for asyncio.'),
      long_description='\n\n'.join((read('README.rst'), read('CHANGES.txt'))),
      classifiers=classifiers,
      platforms=['POSIX'],
      author="Nikolay Novik",
      author_email="nickolainovik@gmail.com",
      url='https://github.com/aio-libs/aiomysql',
      download_url='https://pypi.python.org/pypi/aiomysql',
      license='MIT',
      packages=find_packages(),
      install_requires=install_requires,
      extras_require=extras_require,
      include_package_data = True)



import unittest

from tests import base
from tests._testutils import run_until_complete


class TestNextset(base.AIOPyMySQLTestCase):
    def setUp(self):
        super(TestNextset, self).setUp()
        self.con = self.connections[0]

    @run_until_complete
    def test_nextset(self):
        cur = yield from self.con.cursor()
        yield from cur.execute("SELECT 1; SELECT 2;")
        r = yield from cur.fetchall()
        self.assertEqual([(1,)], list(r))

        r = yield from cur.nextset()
        self.assertTrue(r)

        r = yield from cur.fetchall()
        self.assertEqual([(2,)], list(r))
        res = yield from cur.nextset()
        self.assertIsNone(res)

    @run_until_complete
    def test_skip_nextset(self):
        cur = yield from self.con.cursor()
        yield from cur.execute("SELECT 1; SELECT 2;")
        r = yield from cur.fetchall()
        self.assertEqual([(1,)], list(r))

        yield from cur.execute("SELECT 42")
        r = yield from cur.fetchall()
        self.assertEqual([(42,)], list(r))

    @run_until_complete
    def test_ok_and_next(self):
        cur = yield from self.con.cursor()
        yield from cur.execute("SELECT 1; commit; SELECT 2;")
        r = yield from cur.fetchall()
        self.assertEqual([(1,)], list(r))
        res = yield from cur.nextset()
        self.assertTrue(res)
        res = yield from cur.nextset()
        self.assertTrue(res)
        r = yield from cur.fetchall()
        self.assertEqual([(2,)], list(r))
        res = yield from cur.nextset()
        self.assertIsNone(res)

    @unittest.expectedFailure
    @run_until_complete
    def test_multi_cursor(self):
        cur1 = yield from self.con.cursor()
        cur2 = yield from self.con.cursor()

        yield from cur1.execute("SELECT 1; SELECT 2;")
        yield from cur2.execute("SELECT 42")

        r1 = yield from cur1.fetchall()
        r2 = yield from cur2.fetchall()

        self.assertEqual([(1,)], list(r1))
        self.assertEqual([(42,)], list(r2))

        res = yield from cur1.nextset()
        self.assertTrue(res)

        self.assertEqual([(2,)], list(r1))
        res = yield from cur1.nextset()
        self.assertIsNone(res)

        # TODO: How about SSCursor and nextset?
        # It's very hard to implement correctly...



import asyncio
import aiomysql
from aiomysql import connect, sa, Cursor

import os
import unittest

from sqlalchemy import MetaData, Table, Column, Integer, String
from sqlalchemy.schema import DropTable, CreateTable


meta = MetaData()
tbl = Table('sa_tbl', meta,
            Column('id', Integer, nullable=False,
                   primary_key=True),
            Column('name', String(255)))


class TestSAConnection(unittest.TestCase):
    def setUp(self):
        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(None)
        self.host = os.environ.get('MYSQL_HOST', 'localhost')
        self.port = os.environ.get('MYSQL_PORT', 3306)
        self.user = os.environ.get('MYSQL_USER', 'root')
        self.db = os.environ.get('MYSQL_DB', 'test_pymysql')
        self.password = os.environ.get('MYSQL_PASSWORD', '')

    def tearDown(self):
        self.loop.close()

    @asyncio.coroutine
    def connect(self, **kwargs):
        conn = yield from connect(db=self.db,
                                  user=self.user,
                                  password=self.password,
                                  host=self.host,
                                  loop=self.loop,
                                  **kwargs)
        yield from conn.autocommit(True)
        cur = yield from conn.cursor()
        yield from cur.execute("DROP TABLE IF EXISTS sa_tbl")
        yield from cur.execute("CREATE TABLE sa_tbl "
                               "(id serial, name varchar(255))")
        yield from cur.execute("INSERT INTO sa_tbl (name)"
                               "VALUES ('first')")

        yield from cur._connection.commit()
        # yield from cur.close()
        return sa.SAConnection(conn, sa.engine._dialect)

    def test_execute_text_select(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute("SELECT * FROM sa_tbl;")
            self.assertIsInstance(res.cursor, Cursor)
            self.assertEqual(('id', 'name'), res.keys())
            rows = [r for r in res]
            self.assertTrue(res.closed)
            self.assertIsNone(res.cursor)
            self.assertEqual(1, len(rows))
            row = rows[0]
            self.assertEqual(1, row[0])
            self.assertEqual(1, row['id'])
            self.assertEqual(1, row.id)
            self.assertEqual('first', row[1])
            self.assertEqual('first', row['name'])
            self.assertEqual('first', row.name)
            # TODO: fix this
            yield from conn._connection.commit()
        self.loop.run_until_complete(go())

    def test_execute_sa_select(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute(tbl.select())
            self.assertIsInstance(res.cursor, Cursor)
            self.assertEqual(('id', 'name'), res.keys())
            rows = [r for r in res]
            self.assertTrue(res.closed)
            self.assertIsNone(res.cursor)
            self.assertTrue(res.returns_rows)

            self.assertEqual(1, len(rows))
            row = rows[0]
            self.assertEqual(1, row[0])
            self.assertEqual(1, row['id'])
            self.assertEqual(1, row.id)
            self.assertEqual('first', row[1])
            self.assertEqual('first', row['name'])
            self.assertEqual('first', row.name)
            # TODO: fix this
            yield from conn._connection.commit()

        self.loop.run_until_complete(go())

    def test_execute_sa_insert_with_dict(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert(), {"id": 2, "name": "second"})

            res = yield from conn.execute(tbl.select())
            rows = list(res)
            self.assertEqual(2, len(rows))
            self.assertEqual((1, 'first'), rows[0])
            self.assertEqual((2, 'second'), rows[1])

        self.loop.run_until_complete(go())

    def test_execute_sa_insert_with_tuple(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert(), (2, "second"))

            res = yield from conn.execute(tbl.select())
            rows = list(res)
            self.assertEqual(2, len(rows))
            self.assertEqual((1, 'first'), rows[0])
            self.assertEqual((2, 'second'), rows[1])

        self.loop.run_until_complete(go())

    def test_execute_sa_insert_named_params(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert(), id=2, name="second")

            res = yield from conn.execute(tbl.select())
            rows = list(res)
            self.assertEqual(2, len(rows))
            self.assertEqual((1, 'first'), rows[0])
            self.assertEqual((2, 'second'), rows[1])

        self.loop.run_until_complete(go())

    def test_execute_sa_insert_positional_params(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert(), 2, "second")

            res = yield from conn.execute(tbl.select())
            rows = list(res)
            self.assertEqual(2, len(rows))
            self.assertEqual((1, 'first'), rows[0])
            self.assertEqual((2, 'second'), rows[1])

        self.loop.run_until_complete(go())

    def test_scalar(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.scalar(tbl.count())
            self.assertEqual(1, res)

        self.loop.run_until_complete(go())

    def test_scalar_None(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.delete())
            res = yield from conn.scalar(tbl.select())
            self.assertIsNone(res)
            # TODO: fix this
            yield from conn._connection.commit()

        self.loop.run_until_complete(go())

    def test_row_proxy(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute(tbl.select())
            rows = [r for r in res]
            row = rows[0]
            row2 = yield from (yield from conn.execute(tbl.select())).first()
            self.assertEqual(2, len(row))
            self.assertEqual(['id', 'name'], list(row))
            self.assertIn('id', row)
            self.assertNotIn('unknown', row)
            self.assertEqual('first', row.name)
            self.assertEqual('first', row[tbl.c.name])
            with self.assertRaises(AttributeError):
                row.unknown
            self.assertEqual("(1, 'first')", repr(row))
            self.assertEqual((1, 'first'), row.as_tuple())
            self.assertNotEqual((555, 'other'), row.as_tuple())
            self.assertEqual(row2, row)
            self.assertFalse(row2 != row)
            self.assertNotEqual(5, row)
            # TODO: fix this
            yield from conn._connection.commit()

        self.loop.run_until_complete(go())

    def test_insert(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute(tbl.insert().values(name='second'))
            self.assertEqual(1, res.rowcount)
            self.assertEqual(2, res.lastrowid)

        self.loop.run_until_complete(go())

    def test_raw_insert(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(
                "INSERT INTO sa_tbl (name) VALUES ('third')")
            res = yield from conn.execute(tbl.select())
            self.assertEqual(2, res.rowcount)
            self.assertEqual(('id', 'name'), res.keys())
            self.assertTrue(res.returns_rows)

            rows = [r for r in res]
            self.assertEqual(2, len(rows))
            self.assertEqual(2, rows[1].id)
        self.loop.run_until_complete(go())

    def test_raw_insert_with_params(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute(
                "INSERT INTO sa_tbl (id, name) VALUES (%s, %s)",
                2, 'third')
            res = yield from conn.execute(tbl.select())
            self.assertEqual(2, res.rowcount)
            self.assertEqual(('id', 'name'), res.keys())
            self.assertTrue(res.returns_rows)

            rows = [r for r in res]
            self.assertEqual(2, len(rows))
            self.assertEqual(2, rows[1].id)
        self.loop.run_until_complete(go())

    def test_raw_insert_with_params_dict(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute(
                "INSERT INTO sa_tbl (id, name) VALUES (%(id)s, %(name)s)",
                {'id': 2, 'name': 'third'})
            res = yield from conn.execute(tbl.select())
            self.assertEqual(2, res.rowcount)
            self.assertEqual(('id', 'name'), res.keys())
            self.assertTrue(res.returns_rows)

            rows = [r for r in res]
            self.assertEqual(2, len(rows))
            self.assertEqual(2, rows[1].id)
        self.loop.run_until_complete(go())

    def test_raw_insert_with_named_params(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute(
                "INSERT INTO sa_tbl (id, name) VALUES (%(id)s, %(name)s)",
                id=2, name='third')
            res = yield from conn.execute(tbl.select())
            self.assertEqual(2, res.rowcount)
            self.assertEqual(('id', 'name'), res.keys())
            self.assertTrue(res.returns_rows)

            rows = [r for r in res]
            self.assertEqual(2, len(rows))
            self.assertEqual(2, rows[1].id)
        self.loop.run_until_complete(go())

    def test_raw_insert_with_executemany(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            with self.assertRaises(sa.ArgumentError):
                yield from conn.execute(
                    "INSERT INTO sa_tbl (id, name) VALUES (%(id)s, %(name)s)",
                    [(2, 'third'), (3, 'forth')])
        self.loop.run_until_complete(go())

    def test_delete(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()

            res = yield from conn.execute(tbl.delete().where(tbl.c.id == 1))

            self.assertEqual((), res.keys())
            self.assertEqual(1, res.rowcount)
            self.assertFalse(res.returns_rows)
            self.assertTrue(res.closed)
            self.assertIsNone(res.cursor)

        self.loop.run_until_complete(go())

    def test_double_close(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute("SELECT 1")
            res.close()
            self.assertTrue(res.closed)
            self.assertIsNone(res.cursor)
            res.close()
            self.assertTrue(res.closed)
            self.assertIsNone(res.cursor)

        self.loop.run_until_complete(go())

    @unittest.skip("Find out how to close cursor on __del__ method")
    def test_weakrefs(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            self.assertEqual(0, len(conn._weak_results))
            res = yield from conn.execute("SELECT 1")
            self.assertEqual(1, len(conn._weak_results))
            cur = res.cursor
            self.assertFalse(cur.closed)
            # TODO: fix this, how close cursor if result was deleted
            # yield from cur.close()
            del res
            self.assertTrue(cur.closed)
            self.assertEqual(0, len(conn._weak_results))

        self.loop.run_until_complete(go())

    def test_fetchall(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert().values(name='second'))

            res = yield from conn.execute(tbl.select())
            rows = yield from res.fetchall()
            self.assertEqual(2, len(rows))
            self.assertTrue(res.closed)
            self.assertTrue(res.returns_rows)
            self.assertEqual([(1, 'first'), (2, 'second')], rows)

        self.loop.run_until_complete(go())

    def test_fetchall_closed(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert().values(name='second'))

            res = yield from conn.execute(tbl.select())
            res.close()
            with self.assertRaises(sa.ResourceClosedError):
                yield from res.fetchall()

        self.loop.run_until_complete(go())

    def test_fetchall_not_returns_rows(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute(tbl.delete())
            with self.assertRaises(sa.ResourceClosedError):
                yield from res.fetchall()

        self.loop.run_until_complete(go())

    def test_fetchone_closed(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert().values(name='second'))

            res = yield from conn.execute(tbl.select())
            res.close()
            with self.assertRaises(sa.ResourceClosedError):
                yield from res.fetchone()

        self.loop.run_until_complete(go())

    def test_first_not_returns_rows(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute(tbl.delete())
            with self.assertRaises(sa.ResourceClosedError):
                yield from res.first()

        self.loop.run_until_complete(go())

    def test_fetchmany(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert().values(name='second'))

            res = yield from conn.execute(tbl.select())
            rows = yield from res.fetchmany()
            self.assertEqual(1, len(rows))
            self.assertFalse(res.closed)
            self.assertTrue(res.returns_rows)
            self.assertEqual([(1, 'first')], rows)

        self.loop.run_until_complete(go())

    def test_fetchmany_with_size(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert().values(name='second'))

            res = yield from conn.execute(tbl.select())
            rows = yield from res.fetchmany(100)
            self.assertEqual(2, len(rows))
            self.assertFalse(res.closed)
            self.assertTrue(res.returns_rows)
            self.assertEqual([(1, 'first'), (2, 'second')], rows)

        self.loop.run_until_complete(go())

    def test_fetchmany_closed(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert().values(name='second'))

            res = yield from conn.execute(tbl.select())
            res.close()
            with self.assertRaises(sa.ResourceClosedError):
                yield from res.fetchmany()

        self.loop.run_until_complete(go())

    def test_fetchmany_with_size_closed(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            yield from conn.execute(tbl.insert().values(name='second'))

            res = yield from conn.execute(tbl.select())
            res.close()
            with self.assertRaises(sa.ResourceClosedError):
                yield from res.fetchmany(5555)

        self.loop.run_until_complete(go())

    def test_fetchmany_not_returns_rows(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute(tbl.delete())
            with self.assertRaises(sa.ResourceClosedError):
                yield from res.fetchmany()

        self.loop.run_until_complete(go())

    def test_fetchmany_close_after_last_read(self):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()

            res = yield from conn.execute(tbl.select())
            rows = yield from res.fetchmany()
            self.assertEqual(1, len(rows))
            self.assertFalse(res.closed)
            self.assertTrue(res.returns_rows)
            self.assertEqual([(1, 'first')], rows)
            rows2 = yield from res.fetchmany()
            self.assertEqual(0, len(rows2))
            self.assertTrue(res.closed)

        self.loop.run_until_complete(go())

    def test_create_table(self, **kwargs):
        @asyncio.coroutine
        def go():
            conn = yield from self.connect()
            res = yield from conn.execute(DropTable(tbl))
            with self.assertRaises(sa.ResourceClosedError):
                yield from res.fetchmany()

            with self.assertRaises(aiomysql.ProgrammingError):
                yield from conn.execute("SELECT * FROM sa_tbl")

            res = yield from conn.execute(CreateTable(tbl))
            with self.assertRaises(sa.ResourceClosedError):
                yield from res.fetchmany()

            res = yield from conn.execute("SELECT * FROM sa_tbl")
            self.assertEqual(0, len(list(res)))

        self.loop.run_until_complete(go())



import datetime
import unittest
import aiomysql

from tests import base
from tests._testutils import run_until_complete


class TestOldIssues(base.AIOPyMySQLTestCase):
    @run_until_complete
    def test_issue_3(self):
        """ undefined methods datetime_or_None, date_or_None """
        conn = self.connections[0]
        c = yield from conn.cursor()
        yield from c.execute("drop table if exists issue3")
        yield from c.execute(
            "create table issue3 (d date, t time, dt datetime, ts timestamp)")
        try:
            yield from c.execute(
                "insert into issue3 (d, t, dt, ts) values (%s,%s,%s,%s)",
                (None, None, None, None))
            yield from c.execute("select d from issue3")
            r = yield from c.fetchone()
            self.assertEqual(None, r[0])
            yield from c.execute("select t from issue3")
            r = yield from c.fetchone()
            self.assertEqual(None, r[0])
            yield from c.execute("select dt from issue3")
            r = yield from c.fetchone()
            self.assertEqual(None, r[0])
            yield from c.execute("select ts from issue3")
            r = yield from c.fetchone()
            self.assertTrue(isinstance(r[0], datetime.datetime))
        finally:
            yield from c.execute("drop table issue3")

    @run_until_complete
    def test_issue_4(self):
        """ can't retrieve TIMESTAMP fields """
        conn = self.connections[0]
        c = yield from conn.cursor()
        yield from c.execute("drop table if exists issue4")
        yield from c.execute("create table issue4 (ts timestamp)")
        try:
            yield from c.execute("insert into issue4 (ts) values (now())")
            yield from c.execute("select ts from issue4")
            r = yield from c.fetchone()
            self.assertTrue(isinstance(r[0], datetime.datetime))
        finally:
            yield from c.execute("drop table issue4")

    @run_until_complete
    def test_issue_5(self):
        """ query on information_schema.tables fails """
        con = self.connections[0]
        cur = yield from con.cursor()
        yield from cur.execute("select * from information_schema.tables")

    @run_until_complete
    def test_issue_6(self):
        # test for exception: TypeError: ord() expected a character,
        # but string of length 0 found
        conn = yield from self.connect(db='mysql')
        c = yield from conn.cursor()
        self.assertEqual(conn.db, 'mysql')
        yield from c.execute("select * from user")
        yield from conn.ensure_closed()

    @run_until_complete
    def test_issue_8(self):
        """ Primary Key and Index error when selecting data """
        conn = self.connections[0]
        c = yield from conn.cursor()
        yield from c.execute("drop table if exists test")
        yield from c.execute("""CREATE TABLE `test` (`station` int(10) NOT
            NULL DEFAULT '0', `dh`
            datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
            `echeance` int(1) NOT NULL DEFAULT '0', `me` double DEFAULT NULL,
            `mo` double DEFAULT NULL, PRIMARY
            KEY (`station`,`dh`,`echeance`)) ENGINE=MyISAM DEFAULT
            CHARSET=latin1;""")
        try:
            yield from c.execute("SELECT * FROM test")
            self.assertEqual(0, c.rowcount)
            yield from c.execute(
                "ALTER TABLE `test` ADD INDEX `idx_station` (`station`)")
            yield from c.execute("SELECT * FROM test")
            self.assertEqual(0, c.rowcount)
        finally:
            yield from c.execute("drop table test")

    @run_until_complete
    def test_issue_13(self):
        """ can't handle large result fields """
        conn = self.connections[0]
        cur = yield from conn.cursor()
        yield from cur.execute("drop table if exists issue13")
        try:
            yield from cur.execute("create table issue13 (t text)")
            # ticket says 18k
            size = 18 * 1024
            yield from cur.execute("insert into issue13 (t) values (%s)",
                                   ("x" * size,))
            yield from cur.execute("select t from issue13")
            # use assertTrue so that obscenely huge error messages don't print
            r = yield from cur.fetchone()
            self.assertTrue("x" * size == r[0])
        finally:
            yield from cur.execute("drop table issue13")

    @run_until_complete
    def test_issue_15(self):
        """ query should be expanded before perform character encoding """
        conn = self.connections[0]
        c = yield from conn.cursor()
        yield from c.execute("drop table if exists issue15")
        yield from c.execute("create table issue15 (t varchar(32))")
        try:
            yield from c.execute("insert into issue15 (t) values (%s)",
                                 (u'\xe4\xf6\xfc',))
            yield from c.execute("select t from issue15")
            r = yield from c.fetchone()
            self.assertEqual(u'\xe4\xf6\xfc', r[0])
        finally:
            yield from c.execute("drop table issue15")

    @run_until_complete
    def test_issue_16(self):
        """ Patch for string and tuple escaping """
        conn = self.connections[0]
        c = yield from conn.cursor()
        yield from c.execute("drop table if exists issue16")
        yield from c.execute("create table issue16 (name varchar(32) "
                             "primary key, email varchar(32))")
        try:
            yield from c.execute("insert into issue16 (name, email) values "
                                 "('pete', 'floydophone')")
            yield from c.execute("select email from issue16 where name=%s",
                                 ("pete",))
            r = yield from c.fetchone()
            self.assertEqual("floydophone", r[0])
        finally:
            yield from c.execute("drop table issue16")

    @unittest.skip(
        "test_issue_17() requires a custom, legacy MySQL configuration and "
        "will not be run.")
    @run_until_complete
    def test_issue_17(self):
        """ could not connect mysql use passwod """
        conn = self.connections[0]
        host = self.host
        db = self.db
        c = yield from conn.cursor()
        # grant access to a table to a user with a password
        try:
            yield from c.execute("drop table if exists issue17")
            yield from c.execute(
                "create table issue17 (x varchar(32) primary key)")
            yield from c.execute(
                "insert into issue17 (x) values ('hello, world!')")
            yield from c.execute("grant all privileges on %s.issue17 to "
                                 "'issue17user'@'%%' identified by '1234'"
                                 % db)
            yield from conn.commit()

            conn2 = yield from aiomysql.connect(host=host, user="issue17user",
                                                passwd="1234", db=db,
                                                loop=self.loop)
            c2 = yield from conn2.cursor()
            yield from c2.execute("select x from issue17")
            r = yield from c2.fetchone()
            self.assertEqual("hello, world!", r[0])
        finally:
            yield from c.execute("drop table issue17")


class TestNewIssues(base.AIOPyMySQLTestCase):
    @run_until_complete
    def test_issue_34(self):
        try:
            yield from aiomysql.connect(host="localhost", port=1237,
                                        user="root", loop=self.loop)
            self.fail()
        except aiomysql.OperationalError as e:
            self.assertEqual(2003, e.args[0])
        except Exception:
            self.fail()

    @run_until_complete
    def test_issue_33(self):
        conn = yield from self.connect(charset='utf8')
        c = yield from conn.cursor()
        try:
            yield from c.execute(
                b"drop table if exists hei\xc3\x9fe".decode("utf8"))
            yield from c.execute(
                b"create table hei\xc3\x9fe (name varchar(32))".decode("utf8"))
            yield from c.execute(b"insert into hei\xc3\x9fe (name) "
                                 b"values ('Pi\xc3\xb1ata')".
                                 decode("utf8"))
            yield from c.execute(
                b"select name from hei\xc3\x9fe".decode("utf8"))
            r = yield from c.fetchone()
            self.assertEqual(b"Pi\xc3\xb1ata".decode("utf8"), r[0])
        finally:
            yield from c.execute(b"drop table hei\xc3\x9fe".decode("utf8"))

    @unittest.skip("This test requires manual intervention")
    @run_until_complete
    def test_issue_35(self):
        conn = self.connections[0]
        c = yield from conn.cursor()
        print("sudo killall -9 mysqld within the next 10 seconds")
        try:
            yield from c.execute("select sleep(10)")
            self.fail()
        except aiomysql.OperationalError as e:
            self.assertEqual(2013, e.args[0])

    @run_until_complete
    def test_issue_36(self):
        conn = self.connections[0]
        c = yield from conn.cursor()
        # kill connections[0]
        yield from c.execute("show processlist")
        kill_id = None
        rows = yield from c.fetchall()
        for row in rows:
            id = row[0]
            info = row[7]
            if info == "show processlist":
                kill_id = id
                break
        try:
            # now nuke the connection
            yield from conn.kill(kill_id)
            # make sure this connection has broken
            yield from c.execute("show tables")
            self.fail()
        except Exception:
            pass
        # check the process list from the other connection
        try:
            c = yield from self.connections[1].cursor()
            yield from c.execute("show processlist")
            rows = yield from c.fetchall()
            ids = [row[0] for row in rows]

            self.assertFalse(kill_id in ids)
        finally:
            del self.connections[0]

    @run_until_complete
    def test_issue_37(self):
        conn = self.connections[0]
        c = yield from conn.cursor()
        self.assertEqual(1, (yield from c.execute("SELECT @foo")))

        r = yield from c.fetchone()
        self.assertEqual((None,), r)
        self.assertEqual(0, (yield from c.execute("SET @foo = 'bar'")))
        yield from c.execute("set @foo = 'bar'")

    @run_until_complete
    def test_issue_38(self):
        conn = self.connections[0]
        c = yield from conn.cursor()
        # reduced size for most default mysql installs
        datum = "a" * 1024 * 1023

        try:
            yield from c.execute("drop table if exists issue38")
            yield from c.execute(
                "create table issue38 (id integer, data mediumblob)")
            yield from c.execute("insert into issue38 values (1, %s)",
                                 (datum,))
        finally:
            yield from c.execute("drop table issue38")

    @run_until_complete
    def disabled_test_issue_54(self):
        conn = self.connections[0]
        c = yield from conn.cursor()
        yield from c.execute("drop table if exists issue54")
        big_sql = "select * from issue54 where "
        big_sql += " and ".join("%d=%d" % (i, i) for i in range(0, 100000))

        try:
            yield from c.execute(
                "create table issue54 (id integer primary key)")
            yield from c.execute("insert into issue54 (id) values (7)")
            yield from c.execute(big_sql)

            r = yield from c.fetchone()
            self.assertEqual(7, r[0])
        finally:
            yield from c.execute("drop table issue54")


class TestGitHubIssues(base.AIOPyMySQLTestCase):
    @run_until_complete
    def test_issue_66(self):
        """ 'Connection' object has no attribute 'insert_id' """
        conn = self.connections[0]
        c = yield from conn.cursor()
        self.assertEqual(0, conn.insert_id())
        try:
            yield from c.execute("drop table if exists issue66")
            yield from c.execute("create table issue66 (id integer primary "
                                 "key auto_increment, x integer)")
            yield from c.execute("insert into issue66 (x) values (1)")
            yield from c.execute("insert into issue66 (x) values (1)")
            self.assertEqual(2, conn.insert_id())
        finally:
            yield from c.execute("drop table issue66")

    @run_until_complete
    def test_issue_79(self):
        """ Duplicate field overwrites the previous one in the result
        of DictCursor """
        conn = self.connections[0]
        c = yield from conn.cursor(aiomysql.cursors.DictCursor)

        yield from c.execute("drop table if exists a")
        yield from c.execute("drop table if exists b")
        yield from c.execute("""CREATE TABLE a (id int, value int)""")
        yield from c.execute("""CREATE TABLE b (id int, value int)""")

        a = (1, 11)
        b = (1, 22)
        try:
            yield from c.execute("insert into a values (%s, %s)", a)
            yield from c.execute("insert into b values (%s, %s)", b)

            yield from c.execute("SELECT * FROM a inner join b on a.id = b.id")
            r, *_ = yield from c.fetchall()
            self.assertEqual(r['id'], 1)
            self.assertEqual(r['value'], 11)
            self.assertEqual(r['b.value'], 22)
        finally:
            yield from c.execute("drop table a")
            yield from c.execute("drop table b")

    @run_until_complete
    def test_issue_95(self):
        """ Leftover trailing OK packet for "CALL my_sp" queries """
        conn = self.connections[0]
        cur = yield from conn.cursor()
        yield from cur.execute("DROP PROCEDURE IF EXISTS `foo`")
        yield from cur.execute("""CREATE PROCEDURE `foo` ()
        BEGIN
            SELECT 1;
        END""")
        try:
            yield from cur.execute("""CALL foo()""")
            yield from cur.execute("""SELECT 1""")
            r = yield from cur.fetchone()
            self.assertEqual(r[0], 1)
        finally:
            yield from cur.execute("DROP PROCEDURE IF EXISTS `foo`")

    @run_until_complete
    def test_issue_114(self):
        """ autocommit is not set after reconnecting with ping() """
        conn = yield from self.connect(charset="utf8")
        yield from conn.autocommit(False)
        c = yield from conn.cursor()
        yield from c.execute("""select @@autocommit;""")
        r = yield from c.fetchone()
        self.assertFalse(r[0])
        yield from conn.ensure_closed()
        yield from conn.ping()
        yield from c.execute("""select @@autocommit;""")
        r = yield from c.fetchone()
        self.assertFalse(r[0])
        yield from conn.ensure_closed()

        # Ensure autocommit() is still working
        conn = yield from self.connect(charset="utf8")
        c = yield from conn.cursor()
        yield from c.execute("""select @@autocommit;""")
        r = yield from c.fetchone()
        self.assertFalse(r[0])
        yield from conn.ensure_closed()
        yield from conn.ping()
        yield from conn.autocommit(True)
        yield from c.execute("""select @@autocommit;""")
        r = yield from c.fetchone()
        self.assertTrue(r[0])
        yield from conn.ensure_closed()

    @run_until_complete
    def test_issue_175(self):
        """ The number of fields returned by server is read in wrong way """
        conn = self.connections[0]
        cur = yield from conn.cursor()
        for length in (200, 300):
            cols = ', '.join('c{0} integer'.format(i) for i in range(length))
            sql = 'create table test_field_count ({0})'.format(cols)
            try:
                yield from cur.execute(sql)
                yield from cur.execute('select * from test_field_count')
                assert len(cur.description) == length
            finally:
                yield from cur.execute('drop table if exists test_field_count')
